import os
import traceback
import datetime

import pymongo

from cputil import log


collection = None
PAYLOAD_TRANSFORM = None


def get_db():
    # set up the db connection
    conn = pymongo.MongoClient(
        os.environ.get('TQUEUE_DB_HOST', 'localhost'), connect=False)
    return conn[os.environ.get('TQUEUE_DB_NAME', 'tqueue')]


def get_collection(db):
    return db[os.environ.get('TQUEUE_COLLECTION_NAME', 'tqueue')]


def count_queued():
    return get_collection(get_db()).find({'status': 'queued'}).count()


VALID_STATUS = set(['queued', 'received', 'in_progress', 'success', 'failed'])


class UnknownTaskError(Exception):

    def __init__(self, task_id):
        self.task_id = task_id
        super().__init__(self)

    def __str__(self):
        return self.task_id


def update_task(task_id, status, payload=None, db=None):
    if status not in VALID_STATUS:
        raise ValueError('invalid task status "{}", must be one of {}'.format(
            status, VALID_STATUS
        ))
    q = {'_id': task_id}
    up = {'$set': {'status': status}}
    up['$push'] = {'statuslog': [status, datetime.datetime.utcnow()]}
    if not get_collection(db).find(q).count():
        raise UnknownTaskError(task_id)
    if status == 'success':
        if PAYLOAD_TRANSFORM:
            payload = PAYLOAD_TRANSFORM(payload)
        up['$set']['result'] = payload
    elif status == 'failed':
        # payload is required to be an exception in this case
        strerr = '{} {}'.format(str(payload.__class__.__name__), str(payload))
        log.error('task failing in error {} {}'.format(
            strerr, traceback.format_exc()))
        up['$set']['error'] = payload
    get_collection(db).update(q, up)


def set_payload_transform(func):
    global PAYLOAD_TRANSFORM
    PAYLOAD_TRANSFORM = func
