import os
import time
import signal

from tqueue.daemonutils import getpid, is_running
from tqueue.client import get_task


class NotRunningException(Exception):
    pass


def start_daemon(pidfile, logfile):
    os.system('nohup python -m tqueue.daemon -n 2 -p {} > {} 2>&1 &'.format(
        pidfile, logfile))
    for _ in range(0, 6):
        if os.path.exists(pidfile) and is_running(getpid(pidfile)):
            return
        time.sleep(0.5)
    raise NotRunningException('The daemon seemed to not have started')


def kill_daemon(pidfile):
    if getpid(pidfile) is None:
        print('no daemon running with pidfile "{}"'.format(pidfile))
    else:
        os.kill(getpid(pidfile), signal.SIGINT)
        for _ in range(0, 12):
            if not is_running(getpid(pidfile)):
                os.remove(pidfile)
                return
            time.sleep(0.5)


def wait(task_id, maxtime=10):
    start = time.time()
    while True:
        task = get_task(task_id)
        taskname = task['name']
        if task['status'] not in ('in_progress', 'queued', 'received'):
            return task
        if (time.time() - start) > maxtime:
            raise ValueError(
                'wait({}) took > {} seconds'.format(taskname, maxtime))
        time.sleep(1)
